#!/usr/bin/env python
# coding: utf-8

# In[1]:


import numpy as np

# In[2]:


import pandas as pd

# In[3]:


import matplotlib.pyplot as plt

# In[4]:


import seaborn as sns

plt.style.use('seaborn')

# In[5]:


import datetime as dt

# In[6]:


import sklearn

# In[7]:


from sklearn.preprocessing import StandardScaler

# In[8]:


from sklearn.cluster import KMeans
from sklearn.metrics import silhouette_score
from scipy.cluster.hierarchy import linkage
from scipy.cluster.hierarchy import dendrogram
from scipy.cluster.hierarchy import cut_tree

# In[9]:


inven = pd.read_csv("Data/DataSet.csv", sep=",", encoding="ISO-8859-1", header=0)
inven.head()

# In[10]:


inven.shape

# In[11]:


inven.info()

# In[12]:


inven.describe()

# In[13]:


df_null = round(100 * (inven.isnull().sum()) / len(inven), 2)

# In[14]:


df_null

# In[15]:


rfm_f = inven.groupby('Brand')['OrderAmount'].sum()
rfm_f = rfm_f.reset_index()
rfm_f.columns = ['Brand', 'Amont']
rfm_f.head()

# In[16]:


inven['OrderAmount'].value_counts().head()

# In[17]:


rfm_f

# In[18]:


min_date = min(inven['OrderDate'])
min_date

# In[19]:


inven['OrderDate'] = pd.to_datetime(inven['OrderDate'], format='%d.%m.%Y')

# In[20]:


min_date = min(inven['OrderDate'])
min_date

# In[21]:


inven['date'] = min_date - inven['OrderDate']
inven.head()

# In[22]:


rfm_d = inven.groupby('Brand')['date'].max()
rfm_d = rfm_d.reset_index()
rfm_d.columns = ['Brand', 'unpopular']
rfm_d.head()

# In[23]:


rfm_d['unpopular'] = rfm_d['unpopular'].dt.days
rfm_d.head()

# In[24]:


rfm_p = pd.merge(rfm_d, rfm_f, on='Brand', how='inner')
rfm_p.head()

# In[25]:


max_date = max(inven['OrderDate'])
max_date

# In[26]:


inven['OrderDate'] = pd.to_datetime(inven['OrderDate'], format='%d.%m.%Y')

# In[27]:


max_date = max(inven['OrderDate'])
max_date

# In[28]:


inven['date'] = max_date - inven['OrderDate']
inven.head()

# In[29]:


rfm_m = inven.groupby('Brand')['date'].min()
rfm_m = rfm_m.reset_index()
rfm_m.columns = ['Brand', 'popular']
rfm_m.head()

# In[30]:


rfm_m['popular'] = rfm_m['popular'].dt.days
rfm_m.head()

# In[31]:


rfm = pd.merge(rfm_p, rfm_m, on='Brand', how='inner')
rfm.head()

# In[32]:


attributes = ['Amont', 'unpopular', 'popular']
plt.rcParams['figure.figsize'] = [10, 8]
sns.boxplot(data=rfm[attributes], orient="v", palette="Set2", whis=1.5, saturation=1, width=0.7)
plt.title("Outliers Variable Distribution", fontsize=14, fontweight='bold')
plt.ylabel("Range", fontweight='bold')
plt.xlabel("Attributes", fontweight='bold')

# In[33]:


Q1 = rfm.Amont.quantile(0.05)
Q3 = rfm.Amont.quantile(0.95)
IQR = Q3 - Q1
rfm = rfm[(rfm.Amont >= Q1 - 1.5 * IQR) & (rfm.Amont <= Q3 + 1.5 * IQR)]

Q1 = rfm.popular.quantile(0.05)
Q3 = rfm.popular.quantile(0.95)
IQR = Q3 - Q1
rfm = rfm[(rfm.popular >= Q1 - 1.5 * IQR) & (rfm.popular <= Q3 + 1.5 * IQR)]

Q1 = rfm.unpopular.quantile(0.05)
Q3 = rfm.unpopular.quantile(0.95)
IQR = Q3 - Q1
rfm = rfm[(rfm.unpopular >= Q1 - 1.5 * IQR) & (rfm.unpopular <= Q3 + 1.5 * IQR)]

# In[34]:


rfm_df = rfm[['Amont', 'popular', 'unpopular']]

# In[35]:


scaler = StandardScaler()

# In[36]:


rfm_df_scaled = scaler.fit_transform(rfm_df)
rfm_df_scaled.shape

# In[37]:


rfm_df_scaled = pd.DataFrame(rfm_df_scaled)
rfm_df_scaled.columns = ['Amont', 'popular', 'unpopular']
rfm_df_scaled.head()

# In[38]:


kmeans = KMeans(n_clusters=4, max_iter=50)
kmeans.fit(rfm_df_scaled)

# In[39]:


kmeans.labels_

# In[40]:


ssd = []
range_n_clusters = [2, 3, 4, 5, 6, 7, 8]
for num_clusters in range_n_clusters:
    kmeans = KMeans(n_clusters=num_clusters, max_iter=50)
    kmeans.fit(rfm_df_scaled)

    ssd.append(kmeans.inertia_)

# In[41]:


plt.plot(ssd)

# In[42]:


range_n_clusters = [2, 3, 4, 5, 6, 7, 8]

# In[43]:


kmeans = KMeans(n_clusters=num_clusters, max_iter=50)

# In[44]:


kmeans.fit(rfm_df_scaled)

# In[45]:


cluster_labels = kmeans.labels_

# In[46]:


silhouette_avg = silhouette_score(rfm_df_scaled, cluster_labels)
print("For n_clusters={0}, the silhouette score is {1}".format(num_clusters, silhouette_avg))

# In[47]:


kmeans = KMeans(n_clusters=4, max_iter=50)
kmeans.fit(rfm_df_scaled)

# In[48]:


kmeans.labels_

# In[49]:


rfm['Cluster_Id'] = kmeans.labels_
rfm.head()

# In[50]:


attributes = ['Amont', 'popular', 'unpopular']
plt.rcParams['figure.figsize'] = [10, 8]
sns.boxplot(data=rfm[attributes], orient="v", palette="Set2", whis=1.5, saturation=1, width=0.7)
plt.title("Outliers Variable Distribution", fontsize=14, fontweight='bold')
plt.ylabel("Range", fontweight='bold')
plt.xlabel("Attributes", fontweight='bold')

# In[51]:


sns.boxplot(x='Cluster_Id', y='Amont', data=rfm)

# In[52]:


sns.boxplot(x='Cluster_Id', y='popular', data=rfm)

# In[53]:


sns.boxplot(x='Cluster_Id', y='unpopular', data=rfm)

# In[ ]:


mergings = linkage(rfm_df_scaled, method="single", metric='euclidean')
dendrogram(mergings)
# plt.show()

# In[ ]:


mergings = linkage(rfm_df_scaled, method="complete", metric='euclidean')
dendrogram(mergings)
# plt.show()

# In[ ]:


mergings = linkage(rfm_df_scaled, method="average", metric='euclidean')
dendrogram(mergings)
# plt.show()

# In[ ]:


cluster_labels = cut_tree(mergings, n_clusters=3).reshape(-1, )
cluster_labels

# In[ ]:


rfm['Cluster_Labels'] = cluster_labels



aaa = rfm.replace(to_replace=[0,1], value=['Popular','Unpopular'])


getfinal = aaa.head()
print(getfinal)

# In[ ]:



# In[ ]:




