import numpy as np  # linear algebra
import pandas as pd  # data processing, CSV file I/O (e.g. pd.read_csv)
import random as rd  # generating random numbers
import datetime  # manipulating date formats
import matplotlib.pyplot as plt  # basic plotting
import seaborn as sns  # for prettier plots


from statsmodels.tsa.arima_model import ARIMA
from statsmodels.tsa.statespace.sarimax import SARIMAX
from pandas.plotting import autocorrelation_plot
from statsmodels.tsa.stattools import adfuller, acf, pacf, arma_order_select_ic
import statsmodels.formula.api as smf
import statsmodels.tsa.api as smt
import statsmodels.api as sm
import scipy.stats as scs


import warnings

warnings.filterwarnings('ignore')

time = pd.read_csv("Data/DataSet.csv", engine='python' , encoding = 'latin1')


# formatting the date column correctly
time.EXPdate = time.EXPdate.apply(lambda x: datetime.datetime.strptime(x, '%d.%m.%Y'))
# check

monthly_sales = time.groupby(["Brand", "ItemNo", "OrderAmount",
                              "OrderDate", "Description", "EXPdate"])

# monthly_sales.head(20)

x = time.groupby(['Brand']).count()
x = x.sort_values(by='ItemNo', ascending=False)
x = x.iloc[0:10].reset_index()
# x
# #plot
plt.figure(figsize=(8, 4))
ax = sns.barplot(x.Brand, x.ItemNo, alpha=0.8)
plt.title("Items per Category")
plt.ylabel('# of items', fontsize=12)
plt.xlabel('Category', fontsize=12)
# plt.show()
# In[18]:
time['EXPdate'] = pd.to_numeric(time['EXPdate'], errors='coerce')

ts = time.groupby(["ItemNo"])["EXPdate"].sum()
ts.astype('float')
plt.figure(figsize=(50, 20))
plt.title('Total ItemNo of the Expiry Dates')
plt.xlabel('ItemNo')
plt.ylabel('Dates')
plt.plot(ts);

plt.figure(figsize=(16, 6))
plt.plot(ts.rolling(window=12, center=False).mean(), label='Rolling Mean');
plt.plot(ts.rolling(window=12, center=False).std(), label='Rolling sd');
# plt.legend();


def test_stationarity(timeseries):
    # Perform Dickey-Fuller test:
  #  print('Results of Dickey-Fuller Test:')
    dftest = adfuller(timeseries, autolag='AIC')
    dfoutput = pd.Series(dftest[0:4], index=['Test Statistic', 'p-value', '#Lags Used', 'Number of Observations Used'])
    for key, value in dftest[4].items():
        dfoutput['Critical Value (%s)' % key] = value
  #  print(dfoutput)

#    test_stationarity(ts)


from pandas import Series as Series

# create a differenced series
def difference(dataset, interval=1):
    diff = list()
    for i in range(interval, len(dataset)):
        value = dataset[i] - dataset[i - interval]
        diff.append(value)
    return Series(diff)

# invert differenced forecast
def inverse_difference(last_ob, value):
    return value + last_ob

ts = time.groupby(["ItemNo"])["EXPdate"].sum()
ts.astype('float')
plt.figure(figsize=(16, 16))
plt.subplot(311)
plt.title('Original')
plt.xlabel('ItemNo')
plt.ylabel('Dates')
plt.plot(ts)



def tsplot(y, lags=None, figsize=(10, 8), style='bmh', title=''):
    if not isinstance(y, pd.Series):
        y = pd.Series(y)
    with plt.style.context(style):
        fig = plt.figure(figsize=figsize)
        # mpl.rcParams['font.family'] = 'Ubuntu Mono'
        layout = (3, 2)
        ts_ax = plt.subplot2grid(layout, (0, 0), colspan=2)
        acf_ax = plt.subplot2grid(layout, (1, 0))
        pacf_ax = plt.subplot2grid(layout, (1, 1))
        qq_ax = plt.subplot2grid(layout, (2, 0))
        pp_ax = plt.subplot2grid(layout, (2, 1))

        y.plot(ax=ts_ax)
        ts_ax.set_title(title)
        smt.graphics.plot_acf(y, lags=lags, ax=acf_ax, alpha=0.5)
        smt.graphics.plot_pacf(y, lags=lags, ax=pacf_ax, alpha=0.5)
        sm.qqplot(y, line='s', ax=qq_ax)
        qq_ax.set_title('QQ Plot')
        scs.probplot(y, sparams=(y.mean(), y.std()), plot=pp_ax)

        plt.tight_layout()
    return


# In[26]:


# Simulate an AR(1) process with alpha = 0.6
np.random.seed(1)
n_samples = int(1000)
a = 0.6
x = w = np.random.normal(size=n_samples)

for t in range(n_samples):
    x[t] = a * x[t - 1] + w[t]
limit = 12
_ = tsplot(x, lags=limit, title="AR(1)process")

# In[27]:


# Simulate an AR(2) process

n = int(1000)
alphas = np.array([.444, .333])
betas = np.array([0.])

# Python requires us to specify the zero-lag value which is 1
# Also note that the alphas for the AR model must be negated
# We also set the betas for the MA equal to 0 for an AR(p) model
# For more information see the examples at statsmodels.org
ar = np.r_[1, -alphas]
ma = np.r_[1, betas]

ar2 = smt.arma_generate_sample(ar=ar, ma=ma, nsample=n)
_ = tsplot(ar2, lags=12, title="AR(2) process")

# In[28]:


# Simulate an MA(1) process
n = int(1000)
# set the AR(p) alphas equal to 0
alphas = np.array([0.])
betas = np.array([0.8])
# add zero-lag and negate alphas
ar = np.r_[1, -alphas]
ma = np.r_[1, betas]
ma1 = smt.arma_generate_sample(ar=ar, ma=ma, nsample=n)
limit = 12
_ = tsplot(ma1, lags=limit, title="MA(1) process")

# In[29]:


# Simulate MA(2) process with betas 0.6, 0.4
n = int(1000)
alphas = np.array([0.])
betas = np.array([0.6, 0.4])
ar = np.r_[1, -alphas]
ma = np.r_[1, betas]

ma3 = smt.arma_generate_sample(ar=ar, ma=ma, nsample=n)
_ = tsplot(ma3, lags=12, title="MA(2) process")

# In[30]:


# Simulate an ARMA(2, 2) model with alphas=[0.5,-0.25] and betas=[0.5,-0.3]
max_lag = 12

n = int(5000)  # lots of samples to help estimates
burn = int(n / 10)  # number of samples to discard before fit

alphas = np.array([0.8, -0.65])
betas = np.array([0.5, -0.7])
ar = np.r_[1, -alphas]
ma = np.r_[1, betas]

arma22 = smt.arma_generate_sample(ar=ar, ma=ma, nsample=n, burnin=burn)
_ = tsplot(arma22, lags=max_lag, title="ARMA(2,2) process")

# In[31]:


best_aic = np.inf
best_order = None
best_mdl = None


# print('aic: {:6.5f} | order: {}'.format(best_aic, best_order))

# In[32]:


#
# pick best order by aic
# smallest aic value wins
best_aic = np.inf
best_order = None
best_mdl = None

rng = range(39)
for i in rng:
    for j in rng:
        try:
            tmp_mdl = smt.ARMA(new_ts.values, order=(i, j)).fit(method='mle', trend='nc')
            tmp_aic = tmp_mdl.aic
            if tmp_aic < best_aic:
                best_aic = tmp_aic
                best_order = (i, j)
                best_mdl = tmp_mdl
        except:
            continue

# print('aic: {:6.5f} | order: {}'.format(best_aic, best_order))

# In[33]:


ts = time.groupby(["ItemNo"])["EXPdate"].sum()
finalpt = ts.head(50)
print(finalpt)




