import os
import numpy as np
import pandas as pd
import json
import plotly.express as px
from mlxtend.frequent_patterns import apriori
from mlxtend.frequent_patterns import association_rules



transData = pd.read_csv('Data/transactionData.csv')
ass = transData.head()
# print(ass)

transData['ITEM NAME'] = transData['ITEM NAME'].str.strip()  # removes spaces from beginning and end
transData.dropna(axis=0, subset=['Invoice No'], inplace=True)  # removes duplicate invoice
transData['Invoice No'] = transData['Invoice No'].astype('str')  # converting invoice number to be string
data = transData[~transData['Invoice No'].str.contains('C')]  # remove the credit transactions
data.head()

mybasket = (transData.groupby(['Invoice No', 'ITEM NAME'])['QTY.']
                    .sum().unstack().reset_index().fillna(0)
                    .set_index('Invoice No'))
abc = mybasket.head()
# print(abc)


# converting all positive values to 1 and everything else to 0
def my_encode_units(x):
    if x <= 0:
        return 0
    if x >= 1:
        return 1


my_basket_sets = mybasket.applymap(my_encode_units)
my_basket_sets

# TRAINING MODEL
my_basket_sets = my_basket_sets.replace(np.nan, 0)
# Generatig frequent itemsets
my_frequent_itemsets = apriori(my_basket_sets, min_support=0.07, use_colnames=True, verbose=1)
my_frequent_itemsets

# generating rules
my_rules = association_rules(my_frequent_itemsets, metric="lift", min_threshold=1)

# viewing top 100 rules
my_rules.head(100)

# Making Recommendations
# Filtering rules based on condition
rules = my_rules[(my_rules['lift'] >= 3) & (my_rules['confidence'] >= 0.3)]  # pass to front
print(rules) # antecedents # consequents



predata = transData[["Invoice No", "ITEM NAME"]]


print(predata['ITEM NAME'].value_counts())  # pass to front

